package ru.ifmo.control;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.Properties;
import java.util.concurrent.ConcurrentLinkedQueue;

import javax.swing.JComponent;
import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.validation.SchemaFactory;

import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.evelopers.common.exception.CommonException;
import com.evelopers.unimod.runtime.ModelEngine;

import ru.ifmo.control.event.CarEventProvider;
import ru.ifmo.image.EnemyCar;
import ru.ifmo.image.Explosion;
import ru.ifmo.image.RotatableImage;
import ru.ifmo.image.PlayerCar;

/**
 *    .
 */
public class GameData {
	/**
	 *   (ms)
	 */
	public int sleepAmount;
	/**
	 *  
	 */
	public final double p;
	/**
	 *  
	 */
	public final double q;
	/**
	 *  
	 */
	public ConcurrentLinkedQueue<EnemyCar> enemies = new ConcurrentLinkedQueue<EnemyCar>();
	/**
	 *  ,      .
	 */
	private ConcurrentLinkedQueue<EnemyCar> freeEnemies = new ConcurrentLinkedQueue<EnemyCar>();
	/**
	 *  (  ..)
	 */
	Nature nature;
	/**
	 *  
	 */
	public PlayerCar playerCar;
	/**
	 * 
	 */
	public ConcurrentLinkedQueue<Explosion> explosions = new ConcurrentLinkedQueue<Explosion>();	
	/**
	 *  "" 
	 */
	public int invSize;
	/**
	 *     
	 */
	public final int intersectionSize;
	/**
	 *     
	 */
	public final int crashSteps;
	
	/**
	 * x-  ,   {@link #drawPane}
	 */
	public volatile int mouseX;
	/**
	 * y-  ,   {@link #drawPane}
	 */
	public volatile int mouseY;
	
	/**
	 *   .
	 */
	public static volatile int score = 0;
	
	/**
	 *   
	 */
	public JComponent drawPane;
	/**
	 *     
	 */
	protected Properties config;
	
	private boolean drawScore = false;
	private Font font = new Font(null, Font.BOLD, 48);
	private ArrayList<ModelEngine> engines = new ArrayList<ModelEngine>();
	
	/**
	 *  ,    .
	 * @param parent
	 */
	public GameData(JComponent parent) {
		drawPane = parent;
		
		try {
			config = loadConfiguration("configuration/config.xml", (URLClassLoader) getClass().getClassLoader());
		} catch (IOException e) {
			System.out.println("Configuration couldn't be loaded!");
			System.exit(1);
		}
		
		sleepAmount = Integer.parseInt(config.getProperty("sleepAmount"));
		
		p = Double.parseDouble(config.getProperty("probability_p"));
		q = Double.parseDouble(config.getProperty("probability_q"));
		
		intersectionSize = Integer.parseInt(config.getProperty("intersectionSize"));
		crashSteps = Integer.parseInt(config.getProperty("crashSteps")); 
		
		Document document;
		URLClassLoader urlLoader = (URLClassLoader) getClass().getClassLoader();
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			SchemaFactory sfactory = SchemaFactory.newInstance(
				       XMLConstants.W3C_XML_SCHEMA_NS_URI);
			URL shema = urlLoader.findResource("configuration/game.xsd");
			factory.setSchema(sfactory.newSchema(shema));
			
			DocumentBuilder builder = factory.newDocumentBuilder();
			
			URL fileLoc = urlLoader.findResource("configuration/cars.xml");
			document = builder.parse(fileLoc.openStream());
			
			Element root = document.getDocumentElement();
			
			//  
			NamedNodeMap player = root.getElementsByTagName("player").item(0).getAttributes();
			playerCar = createPlayerCar(player);
			
			//  
			NodeList cars = root.getElementsByTagName("car");
			for (int i = 0; i < cars.getLength(); i++) {
				NamedNodeMap car = cars.item(i).getAttributes();
				int n = Integer.parseInt(car.getNamedItem("amount").getNodeValue());
				Class c = Class.forName(car.getNamedItem("class").getNodeValue());
				for (int j = 0; j < n; j++) {
					enemies.add(createEnemyCar(car));					
					engines.add((ModelEngine)c.getMethod("createModelEngine", boolean.class).invoke(null, true));
				}		
			}
			
			nature = new Nature(parent, 
					config.getProperty("natureImagesFolder"), playerCar);
						
			invSize = (int)(Double.parseDouble(config.getProperty("InvisibleRegionSizePercent")) * 
					Math.max(playerCar.getWidth(), playerCar.getHeight()));
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (DOMException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (SAXException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (ParserConfigurationException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (SecurityException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (InvocationTargetException e) {
			e.printStackTrace();
			System.exit(1);
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
			System.exit(1);
		} 
	}
	
	/**
	 *   
	 * @param car 
	 * @return  
	 * @throws DOMException
	 * @throws IOException
	 */
	protected PlayerCar createPlayerCar(NamedNodeMap car) throws DOMException, IOException {
		PlayerCar pc = new PlayerCar(this, car.getNamedItem("image").getNodeValue());
		pc.setMaxSpeed(Double.parseDouble(car.getNamedItem("maxSpeed").getNodeValue()));
		pc.setMaxAngle(Double.parseDouble(car.getNamedItem("maxAngle").getNodeValue()));
		pc.setAcceleration(Double.parseDouble(car.getNamedItem("acceleration").getNodeValue()));
		pc.setLocation(drawPane.getWidth() / 2, drawPane.getHeight() / 2);
		return pc;
	}
	
	/**
	 *   .
	 * @param car 
	 * @return  
	 * @throws DOMException
	 * @throws IOException
	 */
	protected EnemyCar createEnemyCar(NamedNodeMap car) throws DOMException, IOException {
		EnemyCar ec = new EnemyCar(this, car.getNamedItem("image").getNodeValue());
		ec.setMaxSpeed(Double.parseDouble(car.getNamedItem("maxSpeed").getNodeValue()));
		ec.setMaxAngle(Double.parseDouble(car.getNamedItem("maxAngle").getNodeValue()));
		ec.setAcceleration(Double.parseDouble(car.getNamedItem("acceleration").getNodeValue()));
		ec.setRandomLocation();
		return ec;
	}
	
	/**
	 *         .
	 * @return   
	 * @throws CommonException    
	 */
	public ArrayList<ModelEngine> getModelEngines() {
		return engines;
	}
	
	/**
	 *         .
	 */
	public void reallocateCars() {
		playerCar.setLocation(drawPane.getWidth() / 2, drawPane.getHeight() / 2);
		
		for (EnemyCar car: enemies) {
			car.setRandomLocation();
		}
	}
	
	/**
	 *   1     .
	 */
	public void increaseEnemiesAmount() {
		if (!freeEnemies.isEmpty()) {
			EnemyCar car = freeEnemies.remove();
			car.setRandomLocation();
			enemies.add(car);
			CarEventProvider.fireEventLater(car, CarEventProvider.E1);
		}
	}
	
	/**
	 *   <code>n</code>     .
	 * @param n
	 */
	public void increaseEnemiesAmount(int n) {
		for (int i = 0; i < n; i++)
			increaseEnemiesAmount();
	}
	
	/**
	 *    <code>car</code>   .
	 * @param car
	 */
	public void removeEnemyCar(EnemyCar car) {
		CarEventProvider.fireEventLater(car, CarEventProvider.E3);
		enemies.remove(car);
		freeEnemies.add(car);
	}
	
	/**
	 *  ,    
	 * @param g  <code>Graphics2D</code>
	 * @deprecated
	 */
	public void clearDrawArea(Graphics2D g) {
		nature.clearDrawArea(g);
		playerCar.clearDrawArea(g);
		for (RotatableImage img: enemies)
			img.clearDrawArea(g);
	}
	
	/**
	 *     .
	 * @param b true -  , false -   
	 */
	public void setDrawScoreMode(boolean b) {
		drawScore = b;
	}
	
	/**
	 *   
	 * @return true    , false - 
	 */
	public boolean isDrawScoreMode() {
		return drawScore;
	}
	
	/**
	 *   .
	 * @param g  
	 */
	public void paintAll(Graphics2D g) {
		if (!drawScore) {
			nature.paint(g);
	    	playerCar.paint(g);
			for (RotatableImage img: enemies)
				img.paint(g);
			for (Explosion expl: explosions) {
				expl.paint(g);
				if (!expl.isVisible()) explosions.remove(expl);
			}
		} else {
			g.setColor(Color.black);
			g.fillRect(0, 0, drawPane.getWidth(), drawPane.getHeight());
			g.setColor(Color.white);
			g.setFont(font);
			String str = "Score: " + score;
			Rectangle2D textRect = font.getStringBounds(str, g.getFontRenderContext());
			g.drawString(str, (int)(drawPane.getWidth() - textRect.getWidth()) / 2, 
					(int)(drawPane.getHeight() - textRect.getHeight()) / 2);
		}
	}

	/**
	 *       
	 * @param filename  
	 * @param urlLoader  .
	 * @return  
	 * @throws IOException        
	 */
	protected static Properties loadConfiguration(String filename, URLClassLoader urlLoader) throws IOException {
		Properties config = new Properties();
//		URLClassLoader urlLoader;
		if (urlLoader == null) {
			urlLoader = (URLClassLoader)ClassLoader.getSystemClassLoader();
		}
		URL fileLoc = urlLoader.findResource(filename);
		
		InputStream input = fileLoc.openStream();
		try {
			config.loadFromXML(input);
		} finally {
			try {
				input.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return config;
	}
}
