package ru.ifmo.control;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.net.URL;
import java.net.URLClassLoader;

import javax.swing.AbstractAction;
import javax.swing.BoundedRangeModel;
import javax.swing.Box;
import javax.swing.DefaultBoundedRangeModel;
import javax.swing.ImageIcon;
import javax.swing.JApplet;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JSeparator;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.RootPaneContainer;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import ru.ifmo.control.event.ConfigurableTimer;
import ru.ifmo.control.event.UserEventProvider;
import static ru.ifmo.control.DrawPane.data;

/**
 *  .   .
 */
public class GameGUI implements WindowFocusListener {
	
	private AbstractAction startAction = new StartGameAction();
	private AbstractAction pauseAction = new PauseGameAction();
	private AbstractAction stopAction = new StopGameAction();
	protected HelpAction helpAction;
	private BoundedRangeModel delayModel = new DefaultBoundedRangeModel(30, 0, 10, 100);
	
	private static GameGUI instance = null;
	private static Component app = null; //applet or application
	
	public static JTextField scoreField = new JTextField("0", 7);
	
	/**
	 * .
	 */
	private GameGUI() {}
	
	/**
	 *    <code>rpc</code>   ,
	 *        .
	 * @param rpc   
	 */
	protected synchronized static void addComponentsTo(RootPaneContainer rpc) {
		rpc.getRootPane().setJMenuBar(instance.createMainMenu());
		rpc.getContentPane().setLayout(new BorderLayout());
		rpc.getContentPane().add(new DrawPane(), BorderLayout.CENTER);	
		rpc.getContentPane().add(instance.createToolBar(), BorderLayout.PAGE_START);
	}
	
	/**
	 *   .
	 * @return    null,     ,    .
	 */
	public synchronized static JFrame createJFrame() {
		if (instance != null) { 
			return null;
		}
		instance = new GameGUI();
		
		JFrame frame = new JFrame("La Redada") {
			public void dispose() {
				UserEventProvider.fireEventAndWait(UserEventProvider.E4);
				instance.helpAction.dispose();
				super.dispose();
		    }
		};
		app = frame;
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		
		addComponentsTo(frame);
		
		frame.addWindowFocusListener(instance);
		
		frame.pack();
		frame.setLocationRelativeTo(null);
        frame.setResizable(false);
        
		return frame;
	}
	
	/**
	 *     .
	 * @param applet 
	 */
	public synchronized static void initApplet(JApplet applet) {
		if (instance != null) { 
			return;
		}
		instance = new GameGUI();
		app = applet;
		addComponentsTo(applet);
	}
	
	/**
	 *     .
	 * @return  
	 */
	protected JMenuBar createMainMenu() {
		JMenuBar mainMenu = new JMenuBar();
		mainMenu.add(createGameMenu());
		mainMenu.add(createHelpMenu());
		
		return mainMenu;
    }

	/**
	 *     (, , , 
	 * / ,   ).
	 * @return   
	 */
	protected JMenu createGameMenu() {
        JMenu gameMenu = new JMenu("Game");
        gameMenu.setMnemonic(KeyEvent.VK_G);

        gameMenu.add(startAction);
        gameMenu.add(pauseAction);
        gameMenu.add(stopAction);
        
        gameMenu.add(new JSeparator());
        
        gameMenu.add(new DecreaseDelayAction());
        gameMenu.add(new IncreaseDelayAction());
        
        if (app instanceof JFrame) {
	        gameMenu.add(new JSeparator());
	        
	        gameMenu.add(new AbstractAction("Exit") {
				public void actionPerformed(ActionEvent e) {
					((JFrame)app).dispose();
				}       	
	        });
        }

        return gameMenu;
    }
	
	/**
	 *   .
	 * @return  
	 */
	protected JMenu createHelpMenu() {
		JMenu helpMenu = new JMenu("Help");
		helpMenu.setMnemonic(KeyEvent.VK_H);
		
		helpMenu.add(helpAction = new HelpAction(this));
		helpMenu.add(new JSeparator());
		helpMenu.add(new AboutAction());
		
		return helpMenu;
	}
	
	/**
	 *   ,     
	 *    .
	 * @return  
	 */
	protected JToolBar createToolBar() {
		JToolBar toolbar = new JToolBar();
		
		toolbar.add(startAction);
		toolbar.add(pauseAction);
		toolbar.add(stopAction);
		
		// Create JSlider for choice of delay amount
		JSlider delay = new JSlider(delayModel);
		delay.setValue(data.sleepAmount);
		
		final String formatStr = "Delay (%d ms): ";
		final JLabel delayLabel = new JLabel(String.format(formatStr, delayModel.getValue()));
		
		delayModel.setValue(data.sleepAmount);
		delayModel.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				delayLabel.setText(String.format(formatStr, delayModel.getValue()));
				if (!delayModel.getValueIsAdjusting()) {
					data.sleepAmount = delayModel.getValue();				
					ConfigurableTimer.startNewTimer();
				}
			}		
		});

		delay.setToolTipText("Set game delay");
		delay.setMaximumSize(new Dimension(100, delay.getMaximumSize().height));
		toolbar.add(delayLabel);
		toolbar.add(delay);
		
		delayLabel.setLabelFor(delay);	
		Dimension ld = delayLabel.getPreferredSize();
		ld.setSize(ld.width + 50, ld.height);
		delayLabel.setPreferredSize(ld);
		delayLabel.setHorizontalAlignment(SwingConstants.RIGHT);

		// Create field for score
		toolbar.add(Box.createGlue());
		toolbar.add(new JLabel("Score: "));
		toolbar.add(scoreField);
		
		for (Component cmp: toolbar.getComponents()) {
			cmp.setFocusable(false);
		}
		
		scoreField.setHorizontalAlignment(JTextField.RIGHT);
		Dimension fd = app.getPreferredSize();
		fd.setSize(fd.width, 20);
		scoreField.setMaximumSize(fd);
		scoreField.setEditable(false);
		scoreField.setFocusable(true);
		
		return toolbar;
	}
	
	/**
	 *      
	 * @param fileName  
	 * @return 
	 */
	protected ImageIcon getIcon(String fileName) {
		URLClassLoader urlLoader = (URLClassLoader)getClass().getClassLoader();
		URL fileLoc = urlLoader.findResource(fileName);
		return new ImageIcon(fileLoc);
	}
	
	/**
	 *       "About".
	 */
	protected class AboutAction extends AbstractAction {
		
		public AboutAction() {
    		super("About");
    		putValue(SHORT_DESCRIPTION, "About game");
    		putValue(SMALL_ICON, getIcon("images/icon/About16.gif"));
    	}

		public void actionPerformed(ActionEvent e) {
			 JOptionPane.showMessageDialog(null, "La Redada\nEgorov Kirill\nRaikov Pavel", 
		        		"About", JOptionPane.INFORMATION_MESSAGE);
		}		
    }
	
	/**
	 *      .
	 */
	protected class StartGameAction extends AbstractAction {
		
		public StartGameAction() {
    		super("Start");
    		putValue(SHORT_DESCRIPTION, "Start game");
    		putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_X, 0, true));
    		putValue(SMALL_ICON, getIcon("images/icon/Play16.gif"));
    	}

		public void actionPerformed(ActionEvent e) {
			UserEventProvider.fireEventAndWait(UserEventProvider.E1);
		}
    }
	
	/**
	 *     .
	 */
	protected class PauseGameAction extends AbstractAction {
		
		public PauseGameAction() {
    		super("Pause");
    		putValue(SHORT_DESCRIPTION, "Pause game");
    		putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_C, 0, true));
    		putValue(SMALL_ICON, getIcon("images/icon/Pause16.gif"));
    	}

		public void actionPerformed(ActionEvent e) {
			UserEventProvider.fireEventAndWait(UserEventProvider.E2);
		}
    }
	
	/**
	 *     .
	 */
	protected class StopGameAction extends AbstractAction {
		
		public StopGameAction() {
    		super("Stop");
    		putValue(SHORT_DESCRIPTION, "Stop game");
    		putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_V, 0, true));
    		putValue(SMALL_ICON, getIcon("images/icon/Stop16.gif"));
    	}

		public void actionPerformed(ActionEvent e) {
			UserEventProvider.fireEventAndWait(UserEventProvider.E3);
		}
    }
	
	/**
	 *      .
	 */
	protected class IncreaseDelayAction extends AbstractAction {
		
		public IncreaseDelayAction() {
    		super("Increase delay");
    		putValue(SHORT_DESCRIPTION, "Increase game delay");
    		putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_EQUALS, 0));
    	}

		public void actionPerformed(ActionEvent e) {
			delayModel.setValue(Math.min(delayModel.getValue() + 10, delayModel.getMaximum()));
		}
    }

	/**
	 *      .
	 */
	protected class DecreaseDelayAction extends AbstractAction {
		
		public DecreaseDelayAction() {
    		super("Decrease delay");
    		putValue(SHORT_DESCRIPTION, "Decrease game delay");
    		putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_MINUS, 0));
    	}

		public void actionPerformed(ActionEvent e) {
			delayModel.setValue(Math.max(delayModel.getValue() - 10, delayModel.getMinimum()));
		}
    }

	public void windowGainedFocus(WindowEvent e) {
		UserEventProvider.fireEventAndWait(UserEventProvider.E5);
	}

	public void windowLostFocus(WindowEvent e) {
		try {
			UserEventProvider.fireEventAndWait(UserEventProvider.E6);
		} catch (java.lang.IllegalStateException e1) {}
	}

}
