package ru.ifmo.control;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;
import java.security.AccessControlException;

import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.KeyStroke;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLFrameHyperlinkEvent;

/**
 * ,    .    
 *       .
 *      "help.htm".
 */
public class HelpAction extends AbstractAction {

	private JFrame help;
	private JTextPane textPane = new JTextPane();
	private GameGUI parent;
	
	/**
	 *  .
	 * @param parent  
	 */
	public HelpAction(GameGUI parent) {
		super("Help...");
		this.parent = parent;
		
		putValue(SHORT_DESCRIPTION, "Help");
		putValue(ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0));
		putValue(SMALL_ICON, parent.getIcon("images/icon/Help16.gif"));
		
		help = new JFrame("Help");
		help.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		help.getContentPane().add(new JScrollPane(textPane,
	            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
	            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED),
	            BorderLayout.CENTER);
			
	    textPane.setEditable(false);
	    textPane.setPreferredSize(new Dimension(400, 400));
	    
	    URLClassLoader urlLoader = (URLClassLoader)getClass().getClassLoader();
		URL fileLoc = urlLoader.findResource("configuration/help.htm");
	    try {
			textPane.setPage(fileLoc);
		} catch (IOException e) {
			e.printStackTrace();
		}
		textPane.addHyperlinkListener(createHyperLinkListener());
	    help.pack();    	       	    
	}
	
	/**
	 *       .
	 * @return    
	 */
	public HyperlinkListener createHyperLinkListener() {
		return new HyperlinkListener() {
		    public void hyperlinkUpdate(HyperlinkEvent e) {
				if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
				    if (e instanceof HTMLFrameHyperlinkEvent) {
						((HTMLDocument)textPane.getDocument()).processHTMLFrameHyperlinkEvent(
						    (HTMLFrameHyperlinkEvent)e);
				    } else {
				    	try {
				    		String os = System.getProperty("os.name");
				    		if (os.startsWith("Windows")) {
				    			Runtime.getRuntime().exec("cmd /c start " + e.getURL());
				    		} else {
				    			//TODO;
				    			JOptionPane.showMessageDialog(null, e.getURL().toString(), 
						        		"TODO", JOptionPane.INFORMATION_MESSAGE);
				    		}
						} catch (IOException e1) {
							e1.printStackTrace();
						} catch (AccessControlException e1) {
							// TODO;
							JOptionPane.showMessageDialog(null, e.getURL().toString(), 
					        		"TODO", JOptionPane.INFORMATION_MESSAGE);
						}
				    }
				}
		    }
		};
	}

	public void actionPerformed(ActionEvent e) {
		help.setLocationRelativeTo(null);
		help.setVisible(true);
	}
	
	public void dispose() {
		help.dispose();
	}
}
