package ru.ifmo.image;

import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.io.IOException;
import java.util.concurrent.locks.ReentrantReadWriteLock;

import ru.ifmo.control.GameData;

/**
 *  ,      .
 */
public abstract class Car extends RotatableImage {

	/**
	 *   
	 */
	protected double speed = 0;
	/**
	 *   
	 */
	protected double maxSpeed = 2;
	/**
	 *       
	 */
	protected double maxAngle = Math.PI / 50;
	/**
	 * 
	 */
	protected double acceleration = 0.12;
	/**
	 *  
	 */
	protected GameData data;
	/**
	 *   lock-      
	 *   {@link #rectDst}.
	 */
	protected final ReentrantReadWriteLock rwRectLock = new ReentrantReadWriteLock();
	
	/**
	 *  
	 * @param data  
	 * @param filename     
	 * @throws IOException      
	 */
	public Car(GameData data, String filename) throws IOException {
		super(data.drawPane, filename);
		this.data = data;
		createImageRects();
	}

	/**
	 *   -     .
	 * @return 
	 */
	public double getAcceleration() {
		return acceleration;
	}

	/**
	 *    -     .
	 * @param acceleration 
	 */
	public void setAcceleration(double acceleration) {
		this.acceleration = acceleration;
	}
	
	/**
	 *   
	 * @param v  
	 */
	public void setMaxSpeed(double v) {
		maxSpeed = v;
	}
	
	/**
	 *   
	 * @return  
	 */
	public double getMaxSpeed() {
		return maxSpeed;
	}

	/**
	 *        
	 * @return   
	 */
	public double getMaxAngle() {
		return maxAngle;
	}
	
	/**
	 *    
	 * @return
	 */
	public double getSpeed() {
		return speed;
	}

	/**
	 *        
	 * @param maxAngle  
	 */
	public void setMaxAngle(double maxAngle) {
		this.maxAngle = maxAngle;
	}
	
	/**
	 *  
	 */
	public void speedUp() {
		if (speed < maxSpeed) {
			speed += acceleration;
			if (speed >= maxSpeed) {
				speed = maxSpeed;
			}
		}
	}
	
	/**
	 *  
	 */
	public void reduceSpeed() {
		if (speed > 0) {
			speed -= acceleration;
			if (speed <= 0) {
				speed = 0;
			}
		}
	}
	
	/**
	 *    0.
	 */
	public void stopCarImmediately() {
		speed = 0;
	}
	
	/**
	 *        .
	 */
	public void move() {
		move(speed);
	}
	
	/**
	 *  .
	 */
	public abstract void explode();
	
	/**
	 *      <code>alpha</code>, , 
	 *       {@link #maxAngle}   .
	 * @param alpha  
	 */
	public void rotateTo(double alpha) {
		double theta = Math.IEEEremainder(alpha - angle, Math.PI *2);
		
		if (Math.abs(theta) < maxAngle) {
			setDirectionAngle(alpha);
		} else {
			rotate(Math.signum(theta) * maxAngle);
		}
	}
	
	private Point2D[] rectSrc = new Point2D.Double[4];
	/**
	 *      -  .
	 * @see #calculateLocation()
	 * @see #calculateLocation(int)
	 */
	protected Point2D[] rectDst = new Point2D.Double[4];
	/**
	 *   ,      ,
	 *    -  .
	 * @see #calculateLocation()
	 * @see #calculateLocation(int)
	 */
	protected Point2D center = new Point2D.Double();
	
	private AffineTransform at = new AffineTransform();
	
	/**
	 *    {@link #rectSrc}, 
	 *         , 
	 *   .
	 */
	private void createImageRects() {
		for (int i = 0; i < rectSrc.length; i++) {
			rectSrc[i] = new Point2D.Double();
			rectDst[i] = new Point2D.Double();
		}
		
		int d = data.intersectionSize;
		
		rectSrc[0].setLocation(-width / 2 + d, -height / 2 + d);
		rectSrc[1].setLocation(-width / 2 + d, height / 2 - d);
		rectSrc[2].setLocation(width / 2 - d, height / 2 - d);
		rectSrc[3].setLocation(width / 2 - d, - height / 2 + d);
	}
	
	/**
	 *     <code>car</code>.
	 * @param c  
	 * @return true -  , false - 
	 */
	public boolean intersect(Car c) {
		return intersect(c, 0);
	}
	
	/**
	 *     <code>car</code>  <code>steps</code> .
	 * @param c  
	 * @param steps  
	 * @return true -  , false - 
	 */
	public boolean intersect(Car c, int steps) {
		calculateLocation(steps);
		c.calculateLocation(steps);
		
		for (Point2D p: rectDst) {
			if (c.contains(p)) return true;
		}	

		for (Point2D p: c.rectDst) {
			if (contains(p)) return true;
		}
		
		if (c.contains(center)) return true;
		if (contains(c.center)) return true;
		
		return false;
	}
	
	/**
	 *    <code>p</code> .
	 * @param p 
	 * @return true -  , false - 
	 */
	private boolean contains(Point2D p) {
		rwRectLock.readLock().lock();
		try {
			for (int i = 0; i < rectDst.length; i++) {
				double x1 = rectDst[(i + 1) % rectDst.length].getX() - rectDst[i].getX();
				double y1 = rectDst[(i + 1) % rectDst.length].getY() - rectDst[i].getY();
				double x2 = p.getX() - rectDst[i].getX();
				double y2 = p.getY() - rectDst[i].getY();
				if (x1 * y2 - x2 * y1 > 0)
					return false;
			}
			return true;
		} finally {
			rwRectLock.readLock().unlock();
		}		
	}
	
	/**
	 *         {@link #rectDst}
	 */
	protected void calculateLocation() {
		calculateLocation(0);
	}
	
	/**
	 *      <code>steps</code>   
	 *    {@link #rectDst}
	 */
	protected void calculateLocation(int steps) {
		rLock.lock();
		rwRectLock.writeLock().lock();
		try {
			center.setLocation(x + steps * speed * Math.cos(angle), 
					y + steps * speed * Math.sin(angle));
			at.setToTranslation(center.getX(), center.getY());
			at.rotate(angle);
			at.transform(rectSrc, 0, rectDst, 0, rectSrc.length);
		} finally {
			rLock.unlock();
			rwRectLock.writeLock().unlock();
		}		
	}
}
