package ru.ifmo.image;

import java.awt.geom.Point2D;
import java.io.IOException;

import ru.ifmo.control.GameData;

/**
 *  .     
 *  .
 */
public class EnemyCar extends Car {

	private Car playerCar;
	private Point2D player = new Point2D.Double();
	
	/**
	 *  .
	 * @param data  
	 * @param filename     
	 * @throws IOException       
	 */
	public EnemyCar(GameData data, String filename) throws IOException {
		super(data, filename);
		this.playerCar = data.playerCar;
	}
	
	/**
	 *         ,
	 *     .
	 * @param d   
	 */
	public void setRandomLocation() {
		wLock.lock();	
		try {
			double alpha = playerCar.getDirectionAngle();
			player = playerCar.getLocation(player);
			double xSign = Math.signum(Math.cos(alpha));
			double ySign = Math.signum(Math.sin(alpha));
			xSign = (xSign != 0) ? xSign : 1;
			ySign = (ySign != 0) ? ySign : 1;
			double hsize = Math.max(getHeight() / 2, getWidth() / 2);
			x = player.getX()
					+ (hsize + drawPane.getWidth() / 2 + (data.invSize - hsize)
							* Math.random()) * xSign;
			y = player.getY()
					+ (hsize + drawPane.getHeight() / 2 + (data.invSize - hsize)
							* Math.random()) * ySign;
			if (Math.random() < 0.5) {
				x -= drawPane.getWidth() * xSign * Math.random();
			} else {
				y -= drawPane.getHeight() * ySign * Math.random();
			}
			angle = 2.0 * Math.PI * Math.random() - Math.PI;
//			angle = Math.atan2((player.getY() - y), (player.getX() - x));
		} finally {
			wLock.unlock();
		}
		
		for (EnemyCar e: data.enemies) {
			if (!e.equals(this) && e.intersect(this)) {
				setRandomLocation();
				return;
			}
		}
			
	}
	
	/**
	 *      <code>car</code>   .
	 * @param car     
	 * @return true,  ; false - 
	 */
	public boolean willCrash(Car car) {
		return intersect(car, data.crashSteps);
	}
	
	@Override
	protected void transform() {
		rLock.lock();	
		try {
			playerCar.getLocation(player);
			transform.setToTranslation(x - width / 2 + drawPane.getWidth() / 2 - player.getX(), 
					y - height / 2 + drawPane.getHeight() / 2 - player.getY());
			transform.rotate(angle, width / 2, height / 2);
		} finally {
			rLock.unlock();
		}
	}

	@Override
	public void explode() {
		rLock.lock();
		try {
			Explosion expl = new Explosion(data);
			expl.setLocation(x, y);
			data.explosions.add(expl);
			data.removeEnemyCar(this);
		} finally {
			rLock.unlock();
		}		
	}
	
}
