package ru.ifmo.image;

import java.awt.Component;
import java.awt.Graphics2D;

/**
 *  .      .
 */
public class Stone extends RotatableImage {
	
	public static final int maxX = 2000;
	public static final int maxY = 2000;
	
	private Car playerCar;
	
	private boolean painted = false;
	
	/**
	 *  .
	 * @param parent  .
	 * @param playerCar  
	 */
	public Stone(Component parent, Car playerCar) {
		super(parent);
		if (playerCar == null) throw new NullPointerException("playerCar hasn't been initialized yet!");
		this.playerCar = playerCar;
	}
	
	private int tx;
	private int ty;
	
	/**
	 * ,      .
	 * @return true -  , false - 
	 */
	protected boolean isVisible() {
		playerCar.rLock.lock();
		try {
			tx = (int)Math.IEEEremainder(x - playerCar.x, maxX);
			ty = (int)Math.IEEEremainder(y - playerCar.y, maxY);
		} finally {
			playerCar.rLock.unlock();
		}
		
		return tx + width > - drawPane.getWidth() / 2 && 
			tx - width < drawPane.getWidth() / 2 &&
			ty + height > - drawPane.getHeight() / 2 && 
			ty - height < drawPane.getHeight() / 2;
	}
	
	@Override
	public void clearDrawArea(Graphics2D g) {
		if (painted) {
			super.clearDrawArea(g);
		}
	}
	
	@Override
	public void paint(Graphics2D g) {
		if (isVisible()) {
			super.paint(g);
			painted = true;
		} else {
			painted = false;
		}
	}
	
	@Override
	protected void transform() {
		transform.setToTranslation(
			tx - width / 2 + drawPane.getWidth() / 2 , 
			ty - height / 2 + drawPane.getHeight() / 2 
			);
		transform.rotate(angle, width / 2, height / 2);
	}
}
